"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReplyHandler = exports.GitHub = void 0;
const zhin_1 = require("zhin");
class GitHub {
    constructor(plugin, config) {
        this.plugin = plugin;
        this.config = config;
        this.history = Object.create(null);
        this.http = zhin_1.axios.create();
        this.init();
    }
    async init() { }
    async getTokens(params) {
        return this.http.post('https://github.com/login/oauth/access_token', {}, {
            params: {
                client_id: this.config.appId,
                client_secret: this.config.appSecret,
                ...params,
            },
            headers: { Accept: 'application/json' },
            timeout: this.config.requestTimeout,
        });
    }
    async _request(method, url, replyMsg, data, headers) {
        this.plugin.logger.debug(method, url, data);
        return this.http(url, {
            data,
            methods: method,
            headers: {
                accept: 'application/vnd.github.v3+json',
                authorization: `token ${replyMsg.sender.github_accessToken}`,
                ...headers,
            },
            timeout: this.config.requestTimeout,
        });
    }
    async authorize(message, raw_message) {
        const name = await message.prompt.text(raw_message);
        if (name) {
            message.raw_message = `github authorize ${name}`;
            await this.plugin.executeCommand(message);
        }
        else {
            await message.reply('输入超时');
        }
    }
    async request(method, url, message, body, headers) {
        if (message.message_type !== 'group')
            return '只能在群聊中使用';
        if (!message.sender.github_accessToken) {
            return this.authorize(message, '要使用此功能，请对机器人进行授权。输入你的 GitHub 用户名。');
        }
        try {
            return await this._request(method, url, message, body, headers);
        }
        catch (error) {
            if (error.response?.status !== 401)
                throw error;
        }
        try {
            const data = await this.getTokens({
                refresh_token: message.sender.github_refreshToken,
                grant_type: 'refresh_token',
            });
            message.sender.github_accessToken = data.access_token;
            message.sender.github_refreshToken = data.refresh_token;
        }
        catch {
            return this.authorize(message, '令牌已失效，需要重新授权。输入你的 GitHub 用户名。');
        }
        return await this._request(method, url, message, body, headers);
    }
}
exports.GitHub = GitHub;
class ReplyHandler {
    constructor(github, ctxMessage, content) {
        this.github = github;
        this.ctxMessage = ctxMessage;
        this.content = content;
    }
    async request(method, url, message, body, headers) {
        try {
            await this.github.request(method, url, this.ctxMessage, body, headers);
        }
        catch (err) {
            return message;
        }
    }
    link(url) {
        return url;
    }
    react(url) {
        return this.request('POST', url, '发送失败', {
            content: this.content,
        }, {
            accept: 'application/vnd.github.squirrel-girl-preview',
        });
    }
    async transform(source) {
        const [{ type, data: { url, text }, },] = (0, zhin_1.parseFromTemplate)(source);
        return type !== 'image' ? text : `![${text}](${url})`;
    }
    async reply(url, params) {
        return this.request('POST', url, '发送失败', {
            body: await this.transform(this.content),
            ...params,
        });
    }
    base(url) {
        return this.request('PATCH', url, '修改失败', {
            base: this.content,
        });
    }
    merge(url, method) {
        const [title] = this.content.split('\n', 1);
        const message = this.content.slice(title.length);
        return this.request('PUT', url, '操作失败。', {
            merge_method: method,
            commit_title: title.trim(),
            commit_message: message.trim(),
        });
    }
    rebase(url) {
        return this.merge(url, 'rebase');
    }
    squash(url) {
        return this.merge(url, 'squash');
    }
    async close(url, commentUrl) {
        if (this.content)
            await this.reply(commentUrl);
        await this.request('PATCH', url, '操作失败', {
            state: 'closed',
        });
    }
}
exports.ReplyHandler = ReplyHandler;
